@@ -141,17 +141,14 @@ make_palette (GnomeRRLabeler *labeler)
 #define LABEL_WINDOW_PADDING 12
 
 static gboolean
-label_window_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
+label_window_draw_event_cb (GtkWidget *widget, cairo_t *cr, gpointer data)
 {
-	cairo_t *cr;
 	GdkColor *color;
 	GtkAllocation allocation;
 
 	color = g_object_get_data (G_OBJECT (widget), "color");
 	gtk_widget_get_allocation (widget, &allocation);
 
-	cr = gdk_cairo_create (gtk_widget_get_window (widget));
-
 	/* edge outline */
 
 	cairo_set_source_rgb (cr, 0, 0, 0);
@@ -173,8 +170,6 @@ label_window_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer
 			 allocation.height - LABEL_WINDOW_EDGE_THICKNESS * 2);
 	cairo_fill (cr);
 
-	cairo_destroy (cr);
-
 	return FALSE;
 }
 
@@ -198,8 +193,8 @@ create_label_window (GnomeRRLabeler *labeler, GnomeOutputInfo *output, GdkColor
 	 */
 	g_object_set_data (G_OBJECT (window), "color", color);
 
-	g_signal_connect (window, "expose-event",
-			  G_CALLBACK (label_window_expose_event_cb), labeler);
+	g_signal_connect (window, "draw",
+			  G_CALLBACK (label_window_draw_event_cb), labeler);
 
 	if (labeler->config->clone) {
 		/* Keep this string in sync with gnome-control-center/capplets/display/xrandr-capplet.c:get_display_name() */
@@ -287,16 +282,11 @@ gnome_rr_labeler_hide (GnomeRRLabeler *labeler)
 
 	g_return_if_fail (GNOME_IS_RR_LABELER (labeler));
 
-	if (labeler->windows == NULL)
-		return;
-
 	for (i = 0; i < labeler->num_outputs; i++)
 		if (labeler->windows[i] != NULL) {
 			gtk_widget_destroy (labeler->windows[i]);
 			labeler->windows[i] = NULL;
 		}
-	g_free (labeler->windows);
-	labeler->windows = NULL;
 }
 
 void
