@@ -486,8 +486,7 @@ fill_out_screen_info (Display *xdisplay,
     if (info->screen->rr_major_version == 1 && info->screen->rr_minor_version >= 3) {
         gdk_error_trap_push ();
         info->primary = XRRGetOutputPrimary (xdisplay, xroot);
-	gdk_flush ();
-	gdk_error_trap_pop (); /* ignore error */
+	gdk_error_trap_pop_ignored ();
     }
 #endif
 
@@ -746,8 +745,7 @@ gnome_rr_screen_set_size (GnomeRRScreen *screen,
     gdk_error_trap_push ();
     XRRSetScreenSize (screen->xdisplay, screen->xroot,
 		      width, height, mm_width, mm_height);
-    gdk_flush ();
-    gdk_error_trap_pop (); /* ignore error */
+    gdk_error_trap_pop_ignored ();
 #endif
 }
 
